@@ -684,18 +684,16 @@ pub enum ToolConfigFileParseError {
 
 /// Error returned while parsing a [`MaxFail`](crate::config::elements::MaxFail) input.
 #[derive(Clone, Debug, Error)]
-#[error(
-    "unrecognized value for max-fail: {input}\n(hint: expected either a positive integer or \"all\")"
-)]
+#[error("unrecognized value for max-fail: {reason}")]
 pub struct MaxFailParseError {
-    /// The input that failed to parse.
-    pub input: String,
+    /// The reason parsing failed.
+    pub reason: String,
 }
 
 impl MaxFailParseError {
-    pub(crate) fn new(input: impl Into<String>) -> Self {
+    pub(crate) fn new(reason: impl Into<String>) -> Self {
         Self {
-            input: input.into(),
+            reason: reason.into(),
         }
     }
 }
