@@ -18,6 +18,15 @@ use nextest_metadata::MismatchReason;
 use quick_junit::ReportUuid;
 use std::{collections::BTreeMap, fmt, num::NonZero, process::ExitStatus, time::Duration};
 
+/// A reporter event.
+#[derive(Clone, Debug)]
+pub enum ReporterEvent<'a> {
+    /// A periodic tick.
+    Tick,
+
+    /// A test event.
+    Test(Box<TestEvent<'a>>),
+}
 /// A test event.
 ///
 /// Events are produced by a [`TestRunner`](crate::runner::TestRunner) and
@@ -197,6 +206,9 @@ pub enum TestEventKind<'a> {
 
         /// Whether failure outputs are printed out.
         failure_output: TestOutputDisplay,
+
+        /// The current number of running tests.
+        running: usize,
     },
 
     /// A retry has started.
@@ -209,6 +221,9 @@ pub enum TestEventKind<'a> {
 
         /// Data related to retries.
         retry_data: RetryData,
+
+        /// The current number of running tests.
+        running: usize,
     },
 
     /// A test finished running.
@@ -419,6 +434,14 @@ impl StressProgress {
             } => total.checked_sub(*elapsed).map(StressRemaining::Time),
         }
     }
+
+    /// Returns a unique ID for this stress sub-run, consisting of the run ID and stress index.
+    pub fn unique_id(&self, run_id: ReportUuid) -> String {
+        let stress_current = match self {
+            Self::Count { completed, .. } | Self::Time { completed, .. } => *completed,
+        };
+        format!("{}:@stress-{}", run_id, stress_current)
+    }
 }
 
 /// For a stress test, the amount of time or number of stress runs remaining.
@@ -1035,6 +1058,31 @@ impl ExecutionResult {
         }
     }
 
+    /// Returns true if this result represents a test that was terminated by nextest
+    /// (as opposed to failing naturally).
+    ///
+    /// This is used to suppress output spam when immediate termination is active.
+    ///
+    /// TODO: This is a heuristic that checks if the test was terminated by SIGTERM (Unix) or
+    /// job object (Windows). In an edge case, a test could send SIGTERM to itself, which would
+    /// incorrectly be detected as a nextest-initiated termination. A more robust solution would
+    /// track which tests were explicitly sent termination signals by nextest.
+    pub fn is_termination_failure(&self) -> bool {
+        match self {
+            #[cfg(unix)]
+            ExecutionResult::Fail {
+                failure_status: FailureStatus::Abort(AbortStatus::UnixSignal(libc::SIGTERM)),
+                ..
+            } => true,
+            #[cfg(windows)]
+            ExecutionResult::Fail {
+                failure_status: FailureStatus::Abort(AbortStatus::JobObject),
+                ..
+            } => true,
+            _ => false,
+        }
+    }
+
     /// Returns a static string representation of the result.
     pub fn as_static_str(&self) -> &'static str {
         match self {
@@ -1139,6 +1187,9 @@ pub enum CancelReason {
     /// The global timeout was exceeded.
     GlobalTimeout,
 
+    /// A test failed and fail-fast with immediate termination was specified.
+    TestFailureImmediate,
+
     /// A termination signal (on Unix, SIGTERM or SIGHUP) was received.
     Signal,
 
@@ -1156,6 +1207,7 @@ impl CancelReason {
             CancelReason::TestFailure => "test failure",
             CancelReason::ReportError => "reporting error",
             CancelReason::GlobalTimeout => "global timeout",
+            CancelReason::TestFailureImmediate => "test failure",
             CancelReason::Signal => "signal",
             CancelReason::Interrupt => "interrupt",
             CancelReason::SecondSignal => "second signal",
