@@ -10,28 +10,34 @@
 use super::{RunUnitRequest, RunnerTaskState, ShutdownRequest};
 use crate::{
     config::{
-        elements::MaxFail,
+        elements::{MaxFail, TerminateMode},
         scripts::{ScriptId, SetupScriptConfig},
     },
     input::{InputEvent, InputHandler},
     list::{TestInstance, TestInstanceId, TestList},
     reporter::events::{
-        CancelReason, ExecuteStatus, ExecutionStatuses, FinalRunStats, InfoResponse,
+        CancelReason, ExecuteStatus, ExecutionStatuses, FinalRunStats, InfoResponse, ReporterEvent,
         RunFinishedStats, RunStats, StressIndex, StressProgress, StressRunStats, TestEvent,
         TestEventKind,
     },
     runner::{ExecutorEvent, RunUnitQuery, SignalRequest, StressCondition, StressCount},
-    signal::{JobControlEvent, ShutdownEvent, SignalEvent, SignalHandler, SignalInfoEvent},
-    time::{StopwatchSnapshot, StopwatchStart},
+    signal::{
+        JobControlEvent, ShutdownEvent, ShutdownSignalEvent, SignalEvent, SignalHandler,
+        SignalInfoEvent,
+    },
+    time::StopwatchStart,
 };
 use chrono::Local;
 use debug_ignore::DebugIgnore;
 use futures::future::{Fuse, FusedFuture};
 use quick_junit::ReportUuid;
-use std::{collections::BTreeMap, pin::Pin, time::Duration};
-use tokio::sync::{
-    mpsc::{UnboundedReceiver, UnboundedSender, unbounded_channel},
-    oneshot,
+use std::{collections::BTreeMap, env, pin::Pin, time::Duration};
+use tokio::{
+    sync::{
+        mpsc::{UnboundedReceiver, UnboundedSender, unbounded_channel},
+        oneshot,
+    },
+    time::MissedTickBehavior,
 };
 use tracing::debug;
 
@@ -54,13 +60,14 @@ pub(super) struct DispatcherContext<'a, F> {
     running_tests: BTreeMap<TestInstanceId<'a>, ContextTestInstance<'a>>,
     signal_count: Option<SignalCount>,
     stress_cx: DispatcherStressContext,
+    tick_interval: Duration,
     #[cfg(test)]
     disable_signal_3_times_panic: bool,
 }
 
 impl<'a, F> DispatcherContext<'a, F>
 where
-    F: FnMut(TestEvent<'a>) + Send,
+    F: FnMut(ReporterEvent<'a>) + Send,
 {
     #[expect(clippy::too_many_arguments)]
     pub(super) fn new(
@@ -73,6 +80,11 @@ where
         global_timeout: Duration,
         stress_condition: Option<StressCondition>,
     ) -> Self {
+        // Tick every 50ms by default.
+        let tick_interval_ms = env::var("NEXTEST_PROGRESS_TICK_INTERVAL_MS")
+            .ok()
+            .and_then(|interval| interval.parse::<u64>().ok())
+            .unwrap_or(50);
         Self {
             callback: DebugIgnore(callback),
             run_id,
@@ -89,6 +101,7 @@ where
             running_tests: BTreeMap::new(),
             signal_count: None,
             stress_cx: DispatcherStressContext::new(stress_condition),
+            tick_interval: Duration::from_millis(tick_interval_ms),
             #[cfg(test)]
             disable_signal_3_times_panic: false,
         }
@@ -115,11 +128,18 @@ where
         let mut global_timeout_sleep =
             std::pin::pin!(crate::time::pausable_sleep(self.global_timeout));
 
+        // This is the interval at which tick events are sent to the reporter.
+        let mut tick_interval = tokio::time::interval(self.tick_interval);
+        tick_interval.set_missed_tick_behavior(MissedTickBehavior::Skip);
+
         loop {
             let internal_event = tokio::select! {
                 _ = &mut global_timeout_sleep => {
                     InternalEvent::GlobalTimeout
                 },
+                _ = tick_interval.tick() => {
+                    InternalEvent::Tick
+                },
                 internal_event = executor_rx.recv() => {
                     match internal_event {
                         Some(event) => InternalEvent::Executor(event),
@@ -315,7 +335,7 @@ where
                             // The global timeout has expired, causing cancellation to begin.
                             self.broadcast_request(RunUnitRequest::Signal(
                                 SignalRequest::Shutdown(ShutdownRequest::Once(
-                                    ShutdownEvent::GLOBAL_TIMEOUT,
+                                    ShutdownEvent::TERMINATE,
                                 )),
                             ));
                         }
@@ -338,7 +358,29 @@ where
         }
     }
 
-    pub(super) fn run_started(&mut self, test_list: &'a TestList) {
+    pub(super) fn run_started(&mut self, test_list: &'a TestList, test_threads: usize) {
+        let (stress_count, stress_infinite, stress_duration_nanos) =
+            match self.stress_cx.condition() {
+                Some(StressCondition::Count(StressCount::Count(count))) => {
+                    (Some(count.get()), false, None)
+                }
+                Some(StressCondition::Count(StressCount::Infinite)) => (None, true, None),
+                Some(StressCondition::Duration(duration)) => {
+                    (None, false, Some(duration.as_nanos() as u64))
+                }
+                None => (None, false, None),
+            };
+        crate::fire_usdt!(UsdtRunStart {
+            run_id: self.run_id,
+            profile_name: self.profile_name.clone(),
+            total_tests: test_list.test_count(),
+            filter_count: test_list.run_count(),
+            test_threads,
+            stress_count,
+            stress_infinite,
+            stress_duration_nanos,
+        });
+
         self.basic_callback(TestEventKind::RunStarted {
             test_list,
             run_id: self.run_id,
@@ -359,6 +401,34 @@ where
             ..Default::default()
         };
 
+        // Fire the USDT probe for stress sub-run start.
+        let (stress_current, stress_total) = match &progress {
+            StressProgress::Count {
+                total,
+                completed,
+                elapsed: _,
+            } => {
+                let total = match total {
+                    StressCount::Count(n) => Some(n.get()),
+                    StressCount::Infinite => None,
+                };
+                (*completed, total)
+            }
+            StressProgress::Time {
+                total: _,
+                elapsed: _,
+                completed,
+            } => (*completed, None),
+        };
+        crate::fire_usdt!(UsdtStressSubRunStart {
+            stress_sub_run_id: progress.unique_id(self.run_id),
+            run_id: self.run_id,
+            profile_name: self.profile_name.clone(),
+            stress_current,
+            stress_total,
+            elapsed_nanos: self.stopwatch.snapshot().active.as_nanos() as u64,
+        });
+
         self.basic_callback(TestEventKind::StressSubRunStarted { progress })
     }
 
@@ -370,6 +440,39 @@ where
             .stress_progress()
             .expect("stress_sub_run_finished called in non-stress test context");
 
+        // Fire the USDT probe for stress sub-run done.
+        let (stress_current, stress_total) = match &progress {
+            StressProgress::Count {
+                total,
+                completed,
+                elapsed: _,
+            } => {
+                let total = match total {
+                    StressCount::Count(n) => Some(n.get()),
+                    StressCount::Infinite => None,
+                };
+                (*completed - 1, total)
+            }
+            StressProgress::Time {
+                total: _,
+                elapsed: _,
+                completed,
+            } => (*completed - 1, None),
+        };
+        crate::fire_usdt!(UsdtStressSubRunDone {
+            stress_sub_run_id: progress.unique_id(self.run_id),
+            run_id: self.run_id,
+            profile_name: self.profile_name.clone(),
+            stress_current,
+            stress_total,
+            elapsed_nanos: self.stopwatch.snapshot().active.as_nanos() as u64,
+            sub_run_duration_nanos: sub_elapsed.as_nanos() as u64,
+            total_tests: self.run_stats.initial_run_count,
+            passed: self.run_stats.passed,
+            failed: self.run_stats.failed_count(),
+            skipped: self.run_stats.skipped,
+        });
+
         self.basic_callback(TestEventKind::StressSubRunFinished {
             progress,
             sub_elapsed,
@@ -402,7 +505,7 @@ where
             elapsed: snapshot.active,
             kind,
         };
-        (self.callback)(event)
+        (self.callback)(ReporterEvent::Test(Box::new(event)))
     }
 
     #[inline]
@@ -413,6 +516,10 @@ where
 
     fn handle_event(&mut self, event: InternalEvent<'a>) -> HandleEventResponse {
         match event {
+            InternalEvent::Tick => {
+                (self.callback)(ReporterEvent::Tick);
+                HandleEventResponse::None
+            }
             InternalEvent::Executor(ExecutorEvent::SetupScriptStarted {
                 stress_index,
                 script_id,
@@ -492,7 +599,7 @@ where
 
                 // Fire the setup-script-done probe, extracting the exit code
                 // from the result if available.
-                let _exit_code = match status.result {
+                let exit_code = match status.result {
                     crate::reporter::events::ExecutionResult::Fail {
                         failure_status: crate::reporter::events::FailureStatus::ExitCode(code),
                         ..
@@ -500,6 +607,14 @@ where
                     _ => None,
                 };
 
+                // Extract stdout and stderr lengths from the output
+                let (stdout_len, stderr_len) = match &status.output {
+                    crate::test_output::ChildExecutionOutput::Output { output, .. } => {
+                        output.stdout_stderr_len()
+                    }
+                    crate::test_output::ChildExecutionOutput::StartError(_) => (None, None),
+                };
+
                 crate::fire_usdt!(UsdtSetupScriptDone {
                     id: script_id.unique_id(self.run_id, stress_index.map(|s| s.current)),
                     run_id: self.run_id,
@@ -507,10 +622,12 @@ where
                     program: program.clone(),
                     args: config.command.args.clone(),
                     result: status.result.as_static_str(),
-                    exit_code: _exit_code,
+                    exit_code,
                     duration_nanos: status.time_taken.as_nanos() as u64,
                     stress_current: stress_index.map(|s| s.current),
                     stress_total: stress_index.and_then(|s| s.total.map(|t| t.get())),
+                    stdout_len,
+                    stderr_len,
                 });
 
                 // Setup scripts failing always cause the entire test run to be cancelled
@@ -611,6 +728,7 @@ where
                     failure_output,
                     run_status,
                     delay_before_next_attempt,
+                    running: self.running_tests.len(),
                 })
             }
             InternalEvent::Executor(ExecutorEvent::RetryStarted {
@@ -638,6 +756,7 @@ where
                     stress_index,
                     test_instance,
                     retry_data,
+                    running: self.running_tests.len(),
                 })
             }
             InternalEvent::Executor(ExecutorEvent::Finished {
@@ -652,8 +771,9 @@ where
                 let run_statuses = self.finish_test(test_instance.id(), last_run_status);
                 self.run_stats.on_test_finished(&run_statuses);
 
-                // should this run be cancelled because of a failure?
-                let fail_cancel = self.max_fail.is_exceeded(self.run_stats.failed_count());
+                // Check if this run should be cancelled because of a failure.
+                // is_exceeded returns Some(terminate_mode) if max-fail is exceeded.
+                let terminate_mode = self.max_fail.is_exceeded(self.run_stats.failed_count());
 
                 self.basic_callback(TestEventKind::TestFinished {
                     stress_index,
@@ -667,9 +787,28 @@ where
                     running: self.running(),
                 });
 
-                if fail_cancel {
+                if let Some(terminate_mode) = terminate_mode {
                     // A test failed: start cancellation if required.
-                    self.begin_cancel(CancelReason::TestFailure, CancelEvent::TestFailure)
+                    // Check if we should terminate immediately or wait for running tests.
+                    match terminate_mode {
+                        TerminateMode::Immediate => {
+                            // Terminate running tests immediately.
+                            self.broadcast_request(RunUnitRequest::Signal(
+                                SignalRequest::Shutdown(ShutdownRequest::Once(
+                                    ShutdownEvent::TestFailureImmediate,
+                                )),
+                            ));
+                            self.begin_cancel(
+                                CancelReason::TestFailureImmediate,
+                                CancelEvent::Signal(ShutdownRequest::Once(
+                                    ShutdownEvent::TestFailureImmediate,
+                                )),
+                            )
+                        }
+                        TerminateMode::Wait => {
+                            self.begin_cancel(CancelReason::TestFailure, CancelEvent::TestFailure)
+                        }
+                    }
                 } else {
                     HandleEventResponse::None
                 }
@@ -822,8 +961,15 @@ where
     fn handle_signal_event(&mut self, event: SignalEvent) -> HandleEventResponse {
         match event {
             SignalEvent::Shutdown(event) => {
-                let signal_count = self.increment_signal_count();
-                let req = signal_count.to_request(event);
+                // TestFailureImmediate doesn't participate in signal count escalation.
+                // It can only happen once and doesn't escalate to Twice on repetition.
+                let req = match event {
+                    ShutdownEvent::TestFailureImmediate => ShutdownRequest::Once(event),
+                    ShutdownEvent::Signal(_) => {
+                        let signal_count = self.increment_signal_count();
+                        signal_count.to_request(event)
+                    }
+                };
                 let cancel_reason = event_to_cancel_reason(event);
 
                 self.begin_cancel(cancel_reason, CancelEvent::Signal(req))
@@ -937,21 +1083,42 @@ where
         }
     }
 
-    pub(super) fn run_finished(&mut self) -> StopwatchSnapshot {
+    pub(super) fn run_finished(&mut self) {
         let stopwatch_end = self.stopwatch.snapshot();
+
+        let stress_stats = self.stress_cx.run_stats(self.run_stats.summarize_final());
+        let (stress_completed, stress_success, stress_failed) = match &stress_stats {
+            Some(stats) => (
+                Some(stats.completed.current),
+                Some(stats.success_count),
+                Some(stats.failed_count),
+            ),
+            None => (None, None, None),
+        };
+
+        crate::fire_usdt!(UsdtRunDone {
+            run_id: self.run_id,
+            profile_name: self.profile_name.clone(),
+            total_tests: self.run_stats.initial_run_count,
+            passed: self.run_stats.passed,
+            failed: self.run_stats.failed_count(),
+            skipped: self.run_stats.skipped,
+            duration_nanos: stopwatch_end.active.as_nanos() as u64,
+            paused_nanos: stopwatch_end.paused.as_nanos() as u64,
+            stress_completed,
+            stress_success,
+            stress_failed,
+        });
+
         self.basic_callback(TestEventKind::RunFinished {
             start_time: stopwatch_end.start_time.fixed_offset(),
             run_id: self.run_id,
             elapsed: stopwatch_end.active,
-            run_stats: self
-                .stress_cx
-                .run_stats(self.run_stats.summarize_final())
-                .map_or_else(
-                    || RunFinishedStats::Single(self.run_stats),
-                    RunFinishedStats::Stress,
-                ),
-        });
-        stopwatch_end
+            run_stats: stress_stats.map_or_else(
+                || RunFinishedStats::Single(self.run_stats),
+                RunFinishedStats::Stress,
+            ),
+        })
     }
 
     pub(super) fn run_stats(&self) -> RunStats {
@@ -1124,9 +1291,14 @@ impl DispatcherStressContext {
 
 fn event_to_cancel_reason(event: ShutdownEvent) -> CancelReason {
     match event {
-        #[cfg(unix)]
-        ShutdownEvent::Hangup | ShutdownEvent::Term | ShutdownEvent::Quit => CancelReason::Signal,
-        ShutdownEvent::Interrupt => CancelReason::Interrupt,
+        ShutdownEvent::Signal(sig) => match sig {
+            #[cfg(unix)]
+            ShutdownSignalEvent::Hangup | ShutdownSignalEvent::Term | ShutdownSignalEvent::Quit => {
+                CancelReason::Signal
+            }
+            ShutdownSignalEvent::Interrupt => CancelReason::Interrupt,
+        },
+        ShutdownEvent::TestFailureImmediate => CancelReason::TestFailureImmediate,
     }
 }
 
@@ -1169,6 +1341,7 @@ impl ContextTestInstance<'_> {
 #[expect(clippy::large_enum_variant)]
 #[derive(Debug)]
 enum InternalEvent<'a> {
+    Tick,
     Executor(ExecutorEvent<'a>),
     Signal(SignalEvent),
     Input(InputEvent),
@@ -1236,8 +1409,13 @@ mod tests {
         // TODO: also test TestFinished and SetupScriptFinished events.
         let events = Mutex::new(Vec::new());
         let mut cx = DispatcherContext::new(
-            |event| {
-                events.lock().unwrap().push(event);
+            |event| match event {
+                ReporterEvent::Test(event) => {
+                    events.lock().unwrap().push(event);
+                }
+                ReporterEvent::Tick => {
+                    // Ignore tick events here.
+                }
             },
             ReportUuid::new_v4(),
             "default",
@@ -1282,27 +1460,77 @@ mod tests {
         let response = cx.handle_event(InternalEvent::ReportCancel);
         assert_noop(response, &events);
 
+        // Save a copy before TestFailureImmediate for later tests.
+        let cx_before_test_failure = cx.clone();
+
+        // Test TestFailureImmediate after ReportCancel - should upgrade.
+        let response = cx.handle_event(InternalEvent::Signal(SignalEvent::Shutdown(
+            ShutdownEvent::TestFailureImmediate,
+        )));
+        assert_eq!(
+            response,
+            HandleEventResponse::Cancel(CancelEvent::Signal(ShutdownRequest::Once(
+                ShutdownEvent::TestFailureImmediate
+            ))),
+            "expected TestFailureImmediate"
+        );
+        {
+            let mut events = events.lock().unwrap();
+            assert_eq!(events.len(), 1, "expected 1 event");
+            let event = events.pop().unwrap();
+            let TestEventKind::RunBeginCancel {
+                setup_scripts_running,
+                current_stats,
+                running,
+            } = event.kind
+            else {
+                panic!("expected RunBeginCancel event, found {:?}", event.kind);
+            };
+            assert_eq!(setup_scripts_running, 0, "expected 0 setup scripts running");
+            assert_eq!(running, 0, "expected 0 tests running");
+            assert_eq!(
+                current_stats.cancel_reason,
+                Some(CancelReason::TestFailureImmediate),
+                "expected test failure immediate"
+            );
+        }
+
+        // Send another TestFailureImmediate, ensuring it's ignored (no escalation like signals).
+        let response = cx.handle_event(InternalEvent::Signal(SignalEvent::Shutdown(
+            ShutdownEvent::TestFailureImmediate,
+        )));
+        assert_noop(response, &events);
+
+        // Send a report error after TestFailureImmediate, ensuring it's ignored.
+        let response = cx.handle_event(InternalEvent::ReportCancel);
+        assert_noop(response, &events);
+
         // The rules:
         // * Any one signal will cause that signal.
         // * Any two signals received will cause a SIGKILL.
         // * After a signal is received, any less-important cancel-worthy events
         //   are ignored.
+        // * TestFailureImmediate acts like a signal but doesn't escalate on repetition.
         //
         // Interestingly, this state machine appears to function on Windows too
         // (though of course the only variant is an Interrupt so this only runs
         // one iteration.) Should it be different? No compelling reason to be
         // yet.
-        for sig1 in ShutdownEvent::ALL_VARIANTS {
-            for sig2 in ShutdownEvent::ALL_VARIANTS {
+        for sig1 in ShutdownSignalEvent::ALL_VARIANTS {
+            for sig2 in ShutdownSignalEvent::ALL_VARIANTS {
                 eprintln!("** testing {sig1:?} -> {sig2:?}");
                 // Separate test for each signal to avoid mixing up state.
                 let mut cx = cx.clone();
 
                 // First signal.
-                let response = cx.handle_event(InternalEvent::Signal(SignalEvent::Shutdown(*sig1)));
+                let response = cx.handle_event(InternalEvent::Signal(SignalEvent::Shutdown(
+                    ShutdownEvent::Signal(*sig1),
+                )));
                 assert_eq!(
                     response,
-                    HandleEventResponse::Cancel(CancelEvent::Signal(ShutdownRequest::Once(*sig1))),
+                    HandleEventResponse::Cancel(CancelEvent::Signal(ShutdownRequest::Once(
+                        ShutdownEvent::Signal(*sig1)
+                    ))),
                     "expected Once"
                 );
                 {
@@ -1321,7 +1549,7 @@ mod tests {
                     assert_eq!(running, 0, "expected 0 tests running");
                     assert_eq!(
                         current_stats.cancel_reason,
-                        Some(event_to_cancel_reason(*sig1)),
+                        Some(event_to_cancel_reason(ShutdownEvent::Signal(*sig1))),
                         "expected signal"
                     );
                 }
@@ -1331,7 +1559,9 @@ mod tests {
                 assert_noop(response, &events);
 
                 // Second signal.
-                let response = cx.handle_event(InternalEvent::Signal(SignalEvent::Shutdown(*sig2)));
+                let response = cx.handle_event(InternalEvent::Signal(SignalEvent::Shutdown(
+                    ShutdownEvent::Signal(*sig2),
+                )));
                 assert_eq!(
                     response,
                     HandleEventResponse::Cancel(CancelEvent::Signal(ShutdownRequest::Twice)),
@@ -1361,12 +1591,120 @@ mod tests {
                 // Another report error, ensuring it's ignored.
                 let response = cx.handle_event(InternalEvent::ReportCancel);
                 assert_noop(response, &events);
+
+                // TestFailureImmediate after signal should be ignored (signal is more severe).
+                let response = cx.handle_event(InternalEvent::Signal(SignalEvent::Shutdown(
+                    ShutdownEvent::TestFailureImmediate,
+                )));
+                assert_noop(response, &events);
+            }
+        }
+
+        // Test that signals upgrade from TestFailureImmediate.
+        for sig in ShutdownSignalEvent::ALL_VARIANTS {
+            eprintln!("** testing TestFailureImmediate -> {sig:?}");
+            // Separate test for each signal to avoid mixing up state.
+            // Clone from before TestFailureImmediate was sent.
+            let mut cx = cx_before_test_failure.clone();
+
+            // First, send TestFailureImmediate.
+            let response = cx.handle_event(InternalEvent::Signal(SignalEvent::Shutdown(
+                ShutdownEvent::TestFailureImmediate,
+            )));
+            assert_eq!(
+                response,
+                HandleEventResponse::Cancel(CancelEvent::Signal(ShutdownRequest::Once(
+                    ShutdownEvent::TestFailureImmediate
+                ))),
+                "expected TestFailureImmediate"
+            );
+            {
+                let mut events = events.lock().unwrap();
+                assert_eq!(events.len(), 1, "expected 1 event");
+                let event = events.pop().unwrap();
+                let TestEventKind::RunBeginCancel {
+                    setup_scripts_running,
+                    current_stats,
+                    running,
+                } = event.kind
+                else {
+                    panic!("expected RunBeginCancel event, found {:?}", event.kind);
+                };
+                assert_eq!(setup_scripts_running, 0, "expected 0 setup scripts running");
+                assert_eq!(running, 0, "expected 0 tests running");
+                assert_eq!(
+                    current_stats.cancel_reason,
+                    Some(CancelReason::TestFailureImmediate),
+                    "expected test failure immediate"
+                );
+            }
+
+            // Now send a signal - should upgrade.
+            let response = cx.handle_event(InternalEvent::Signal(SignalEvent::Shutdown(
+                ShutdownEvent::Signal(*sig),
+            )));
+            assert_eq!(
+                response,
+                HandleEventResponse::Cancel(CancelEvent::Signal(ShutdownRequest::Once(
+                    ShutdownEvent::Signal(*sig)
+                ))),
+                "expected signal upgrade"
+            );
+            {
+                let mut events = events.lock().unwrap();
+                assert_eq!(events.len(), 1, "expected 1 event");
+                let event = events.pop().unwrap();
+                let TestEventKind::RunBeginCancel {
+                    setup_scripts_running,
+                    current_stats,
+                    running,
+                } = event.kind
+                else {
+                    panic!("expected RunBeginCancel event, found {:?}", event.kind);
+                };
+                assert_eq!(setup_scripts_running, 0, "expected 0 setup scripts running");
+                assert_eq!(running, 0, "expected 0 tests running");
+                assert_eq!(
+                    current_stats.cancel_reason,
+                    Some(event_to_cancel_reason(ShutdownEvent::Signal(*sig))),
+                    "expected signal cancel reason"
+                );
+            }
+
+            // A second signal should cause a kill.
+            let response = cx.handle_event(InternalEvent::Signal(SignalEvent::Shutdown(
+                ShutdownEvent::Signal(*sig),
+            )));
+            assert_eq!(
+                response,
+                HandleEventResponse::Cancel(CancelEvent::Signal(ShutdownRequest::Twice)),
+                "expected kill"
+            );
+            {
+                let mut events = events.lock().unwrap();
+                assert_eq!(events.len(), 1, "expected 1 event");
+                let event = events.pop().unwrap();
+                let TestEventKind::RunBeginKill {
+                    setup_scripts_running,
+                    current_stats,
+                    running,
+                } = event.kind
+                else {
+                    panic!("expected RunBeginKill event, found {:?}", event.kind);
+                };
+                assert_eq!(setup_scripts_running, 0, "expected 0 setup scripts running");
+                assert_eq!(running, 0, "expected 0 tests running");
+                assert_eq!(
+                    current_stats.cancel_reason,
+                    Some(CancelReason::SecondSignal),
+                    "expected second signal"
+                );
             }
         }
     }
 
     #[track_caller]
-    fn assert_noop(response: HandleEventResponse, events: &Mutex<Vec<TestEvent<'_>>>) {
+    fn assert_noop(response: HandleEventResponse, events: &Mutex<Vec<Box<TestEvent<'_>>>>) {
         assert_eq!(response, HandleEventResponse::None, "expected no response");
         assert_eq!(events.lock().unwrap().len(), 0, "expected no new events");
     }
